declare function Arr<TElement extends RuntypeBase<unknown>>(element: TElement): Arr<TElement>;

declare interface Arr<E extends RuntypeBase<unknown> = RuntypeBase<unknown>> extends Codec<Static<E>[]> {
    readonly tag: 'array';
    readonly element: E;
    readonly isReadonly: false;
    asReadonly(): ReadonlyArray_2<E>;
}
export { Arr as Array }
export { Arr as MutableArray }

export declare function assertType<TRuntypeBase extends RuntypeBase>(rt: TRuntypeBase, v: unknown): asserts v is Static<TRuntypeBase>;

/**
 * Create a function contract.
 */
export declare function AsyncContract<A extends [any, ...any[]] | [], Z>(argTypes: {
    [key in keyof A]: key extends 'length' ? A['length'] : RuntypeBase<A[key]>;
}, returnType: RuntypeBase<Z>): AsyncContract<A, Z>;

export declare interface AsyncContract<A extends any[], Z> {
    enforce(f: (...a: A) => Promise<Z>): (...a: A) => Promise<Z>;
}

declare interface BigInt_2 extends Codec<bigint> {
    readonly tag: 'bigint';
}

/**
 * Validates that a value is a BigInt.
 */
declare const BigInt_2: BigInt_2;
export { BigInt_2 as BigInt }

declare interface Boolean_2 extends Codec<boolean> {
    readonly tag: 'boolean';
}

/**
 * Validates that a value is a boolean.
 */
declare const Boolean_2: Boolean_2;
export { Boolean_2 as Boolean }

export declare function Brand<B extends string, A extends RuntypeBase<unknown>>(brand: B, entity: A): Brand<B, A>;

export declare interface Brand<B extends string, A extends RuntypeBase<unknown>> extends Codec<Static<A> & {
    [RuntypeName]: B;
}> {
    readonly tag: 'brand';
    readonly brand: B;
    readonly entity: A;
}

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type Case<T extends RuntypeBase<unknown>, Result> = (v: Static<T>) => Result;

export declare interface Codec<TParsed> extends Runtype<TParsed> {
    /**
     * Validates the value conforms to this type, and performs
     * the `serialize` action for any `ParsedValue` types.
     *
     * If the value is valid, and the type supports serialize,
     * it returns the serialized value, otherwise it throws a
     * ValidationError.
     *
     * @throws ValidationError
     */
    serialize: (x: TParsed) => unknown;
    /**
     * Validates the value conforms to this type, and performs
     * the `serialize` action for any `ParsedValue` types.
     *
     * Returns a `Result`, constaining the serialized value or
     * error message. Does not throw!
     */
    safeSerialize: (x: TParsed) => Result<unknown>;
}

export declare function Constraint<TUnderlying extends RuntypeBase<unknown>, TConstrained extends Static<TUnderlying> = Static<TUnderlying>, TArgs = unknown>(underlying: TUnderlying, constraint: ConstraintCheck<TUnderlying>, options?: {
    name?: string;
    args?: TArgs;
}): Constraint<TUnderlying, TConstrained, TArgs>;

export declare interface Constraint<TUnderlying extends RuntypeBase<unknown>, TConstrained extends Static<TUnderlying> = Static<TUnderlying>, TArgs = unknown> extends Codec<TConstrained> {
    readonly tag: 'constraint';
    readonly underlying: TUnderlying;
    constraint(x: Static<TUnderlying>): boolean | string;
    readonly name?: string;
    readonly args?: TArgs;
}

export declare type ConstraintCheck<A extends RuntypeBase<unknown>> = (x: Static<A>) => boolean | string;

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare interface Constructor<V> {
    new (...args: any[]): V;
}

/**
 * Create a function contract.
 */
export declare function Contract<A extends [any, ...any[]] | [], Z>(argTypes: {
    [key in keyof A]: key extends 'length' ? A['length'] : RuntypeBase<A[key]>;
}, returnType: RuntypeBase<Z>): Contract<A, Z>;

export declare interface Contract<A extends any[], Z> {
    enforce(f: (...a: A) => Z): (...a: A) => Z;
}

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare interface Cycle<T> {
    success: true;
    cycle: true;
    placeholder: Partial<T>;
    unwrap: () => Result<T>;
}

export declare function Enum<TEnum extends {
    [key: string]: number | string;
}>(name: string, e: TEnum): Enum<TEnum>;

export declare interface Enum<TEnum extends {
    [key: string]: number | string;
}> extends Codec<TEnum[keyof TEnum]> {
    readonly tag: 'enum';
    readonly enumObject: TEnum;
}

/**
 * A failed validation result.
 */
export declare type Failure = {
    /**
     * A tag indicating failure.
     */
    success: false;
    /**
     * A message indicating the reason validation failed.
     */
    message: string;
    fullError?: FullError;
    /**
     * A key indicating the location at which validation failed.
     */
    key?: string;
};

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type FullError = [string, ...FullError[]];

declare interface Function_2 extends Codec<(...args: any[]) => any> {
    readonly tag: 'function';
}

/**
 * Validates that a value is a function.
 */
declare const Function_2: Function_2;
export { Function_2 as Function }

export declare interface Guard<TConstrained, TArgs = unknown> extends Constraint<Unknown, TConstrained, TArgs> {
}

export declare const Guard: <T, K = unknown>(test: (x: unknown) => x is T, options?: {
    name?: string | undefined;
    args?: K | undefined;
} | undefined) => Guard<T, K>;

export declare function InstanceOf<V>(ctor: Constructor<V>): InstanceOf<V>;

export declare interface InstanceOf<V = unknown> extends Codec<V> {
    readonly tag: 'instanceof';
    readonly ctor: Constructor<V>;
}

declare const internal: typeof internalSymbol;

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare interface InternalRecord<O extends RecordFields, IsPartial extends boolean, IsReadonly extends boolean> extends Codec<RecordStaticType<O, IsPartial, IsReadonly>> {
    readonly tag: 'object';
    readonly fields: O;
    readonly isPartial: IsPartial;
    readonly isReadonly: IsReadonly;
    asPartial(): Partial_2<O, IsReadonly>;
    asReadonly(): IsPartial extends false ? Obj<O, true> : Partial_2<O, true>;
    pick<TKeys extends [keyof O, ...(keyof O)[]]>(...keys: TKeys): InternalRecord<Pick<O, TKeys[number]>, IsPartial, IsReadonly>;
    omit<TKeys extends [keyof O, ...(keyof O)[]]>(...keys: TKeys): InternalRecord<Omit<O, TKeys[number]>, IsPartial, IsReadonly>;
}

declare const internalSymbol: unique symbol;

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare interface InternalValidation<TParsed> {
    /**
     * parse
     */
    p(x: any, innerValidate: <T>(runtype: RuntypeBase<T>, value: unknown, sealed?: SealedState) => Result<T>, innerValidateToPlaceholder: <T>(runtype: RuntypeBase<T>, value: unknown, sealed?: SealedState) => ResultWithCycle<T>, mode: 'p' | 's' | 't', sealed: SealedState): ResultWithCycle<TParsed>;
    /**
     * test
     */
    t?: (x: any, innerValidate: <T>(runtype: RuntypeBase<T>, value: unknown, sealed?: SealedState) => Failure | undefined, sealed: SealedState, isOptionalTest: boolean) => Failure | undefined;
    /**
     * serialize
     */
    s?: (x: any, innerSerialize: (runtype: RuntypeBase, value: unknown, sealed?: SealedState) => Result<any>, innerSerializeToPlaceholder: (runtype: RuntypeBase, value: unknown, sealed?: SealedState) => ResultWithCycle<any>, mode: 's', sealed: SealedState) => ResultWithCycle<any>;
    /**
     * get underlying type
     */
    u?: (mode: 'p' | 's' | 't') => RuntypeBase | undefined;
    /**
     * get fields, not called if "u" is implemented, can return
     * undefined to indicate that arbitrarily many fields are
     * possible.
     */
    f?: (mode: 'p' | 't' | 's') => ReadonlySet<string> | undefined;
}

/**
 * Construct an intersection runtype from runtypes for its alternatives.
 */
export declare function Intersect<TIntersectees extends readonly [RuntypeBase<unknown>, ...RuntypeBase<unknown>[]]>(...intersectees: TIntersectees): Intersect<TIntersectees>;

export declare interface Intersect<TIntersectees extends readonly [RuntypeBase<unknown>, ...RuntypeBase<unknown>[]]> extends Codec<StaticIntersect<TIntersectees>> {
    readonly tag: 'intersect';
    readonly intersectees: TIntersectees;
}

export declare function KeyOf<TObject extends Object>(object: TObject): KeyOf<TObject>;

export declare interface KeyOf<TObject extends Object> extends Codec<keyof TObject> {
    readonly tag: 'keyOf';
    readonly keys: Set<string>;
}

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type KeyRuntypeBase = KeyRuntypeBaseWithoutUnion | Pick<Union<KeyRuntypeBaseWithoutUnion[]>, 'alternatives' | keyof RuntypeBase>;

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type KeyRuntypeBaseWithoutUnion = Pick<String_2, keyof RuntypeBase> | Pick<Number_2, keyof RuntypeBase> | Pick<Literal<string | number>, 'value' | keyof RuntypeBase> | Pick<Constraint<KeyRuntypeBase, string | number>, 'underlying' | keyof RuntypeBase>;

/**
 * Construct a possibly-recursive Runtype.
 */
export declare function Lazy<TUnderlying extends RuntypeBase<unknown>>(delayed: () => TUnderlying): Lazy<TUnderlying>;

export declare interface Lazy<TUnderlying extends RuntypeBase<unknown>> extends Codec<Static<TUnderlying>> {
    readonly tag: 'lazy';
    readonly underlying: () => TUnderlying;
}

/**
 * Construct a runtype for a type literal.
 */
export declare function Literal<A extends LiteralValue>(valueBase: A): Literal<A>;

export declare interface Literal<TLiteralValue extends LiteralValue = LiteralValue> extends Codec<TLiteralValue> {
    readonly tag: 'literal';
    readonly value: TLiteralValue;
}

/**
 * The super type of all literal types.
 */
export declare type LiteralValue = undefined | null | boolean | number | string;

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare interface Match<A extends readonly RuntypeBase<unknown>[]> {
    <Z>(...a: {
        [key in keyof A]: A[key] extends RuntypeBase<unknown> ? Case<A[key], Z> : never;
    }): Matcher<A, Z>;
}

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type Matcher<A extends readonly RuntypeBase<unknown>[], Z> = (x: {
    [key in keyof A]: A[key] extends RuntypeBase<infer Type> ? Type : unknown;
}[number]) => Z;

export declare function Mutable<TFields extends RecordFields, TPartial extends boolean>(input: InternalRecord<TFields, TPartial, true>): InternalRecord<TFields, TPartial, false>;

export declare function Mutable<TElement extends RuntypeBase>(input: ReadonlyArray_2<TElement>): Arr<TElement>;

export declare function Mutable<TElements extends RuntypeBase<unknown>[] = RuntypeBase<unknown>[]>(input: ReadonlyTuple<TElements>): Tuple<TElements>;

export declare function Mutable<K extends KeyRuntypeBase, V extends RuntypeBase<unknown>>(record: ReadonlyRecord<K, V>): Record_2<K, V>;

export declare type Mutable<T extends RuntypeBase> = T extends InternalRecord<infer TFields, infer TPartial, true> ? InternalRecord<TFields, TPartial, false> : T extends ReadonlyArray_2<infer TElement> ? Arr<TElement> : T extends ReadonlyTuple<infer TElements> ? Tuple<TElements> : T extends ReadonlyRecord<infer K, infer V> ? Record_2<K, V> : unknown;

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type MutableRecordStaticType<O extends RecordFields> = {
    -readonly [K in keyof O]: Static<O[K]>;
};

export declare function Named<TUnderlying extends RuntypeBase<unknown>>(name: string, underlying: TUnderlying): Named<TUnderlying>;

export declare interface Named<TUnderlying extends RuntypeBase<unknown>> extends Codec<Static<TUnderlying>> {
    readonly tag: 'named';
    readonly underlying: TUnderlying;
    readonly name: string;
}

export declare interface Never extends Codec<never> {
    readonly tag: 'never';
}

/**
 * Validates nothing (unknown fails).
 */
export declare const Never: Never;

/**
 * An alias for Literal(null).
 */
export declare const Null: Literal<null>;

declare interface Number_2 extends Codec<number> {
    readonly tag: 'number';
}

/**
 * Validates that a value is a number.
 */
declare const Number_2: Number_2;
export { Number_2 as Number }

declare function Obj<O extends RecordFields>(fields: O): Obj<O, false>;

declare type Obj<O extends RecordFields, IsReadonly extends boolean> = InternalRecord<O, false, IsReadonly>;
export { Obj as MutableObject }
export { Obj as Object }

export declare function ParsedValue<TUnderlying extends RuntypeBase<unknown>, TParsed>(underlying: TUnderlying, config: ParsedValueConfig<TUnderlying, TParsed>): ParsedValue<TUnderlying, TParsed>;

export declare interface ParsedValue<TUnderlying extends RuntypeBase<unknown>, TParsed> extends Codec<TParsed> {
    readonly tag: 'parsed';
    readonly underlying: TUnderlying;
    readonly config: ParsedValueConfig<TUnderlying, TParsed>;
}

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare interface ParsedValueConfig<TUnderlying extends RuntypeBase<unknown>, TParsed> {
    name?: string;
    parse: (value: Static<TUnderlying>) => Result<TParsed>;
    serialize?: (value: TParsed) => Result<Static<TUnderlying>>;
    test?: RuntypeBase<TParsed>;
}

declare function Partial_2<O extends RecordFields>(fields: O): Partial_2<O, false>;

declare type Partial_2<O extends RecordFields, IsReadonly extends boolean> = InternalRecord<O, true, IsReadonly>;
export { Partial_2 as MutablePartial }
export { Partial_2 as Partial }

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type PartialMutableRecordStaticType<O extends RecordFields> = {
    -readonly [K in keyof O]?: Static<O[K]>;
};

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type PartialReadonlyRecordStaticType<O extends RecordFields> = {
    readonly [K in keyof O]?: Static<O[K]>;
};

declare function Readonly_2<TFields extends RecordFields, TPartial extends boolean>(input: InternalRecord<TFields, TPartial, false>): InternalRecord<TFields, TPartial, true>;

declare function Readonly_2<TElement extends RuntypeBase>(input: Arr<TElement>): ReadonlyArray_2<TElement>;

declare function Readonly_2<TElements extends readonly RuntypeBase<unknown>[] = readonly RuntypeBase<unknown>[]>(input: Tuple<TElements>): ReadonlyTuple<TElements>;

declare function Readonly_2<K extends KeyRuntypeBase, V extends RuntypeBase<unknown>>(record: Record_2<K, V>): ReadonlyRecord<K, V>;

declare type Readonly_2<T extends RuntypeBase> = T extends InternalRecord<infer TFields, infer TPartial, false> ? InternalRecord<TFields, TPartial, true> : T extends Arr<infer TElement> ? ReadonlyArray_2<TElement> : T extends Tuple<infer TElements> ? ReadonlyTuple<TElements> : T extends Record_2<infer K, infer V> ? ReadonlyRecord<K, V> : unknown;
export { Readonly_2 as Readonly }

declare function ReadonlyArray_2<TElement extends RuntypeBase<unknown>>(element: TElement): ReadonlyArray_2<TElement>;

declare interface ReadonlyArray_2<E extends RuntypeBase<unknown> = RuntypeBase<unknown>> extends Codec<readonly Static<E>[]> {
    readonly tag: 'array';
    readonly element: E;
    readonly isReadonly: true;
}
export { ReadonlyArray_2 as ReadonlyArray }

export declare function ReadonlyObject<O extends RecordFields>(fields: O): Obj<O, true>;

export declare function ReadonlyPartial<O extends RecordFields>(fields: O): Partial_2<O, true>;

export declare function ReadonlyRecord<K extends KeyRuntypeBase, V extends RuntypeBase<unknown>>(key: K, value: V): ReadonlyRecord<K, V>;

export declare interface ReadonlyRecord<K extends KeyRuntypeBase, V extends RuntypeBase<unknown>> extends Codec<{
    readonly [_ in Static<K>]?: Static<V>;
}> {
    readonly tag: 'record';
    readonly key: K;
    readonly value: V;
    readonly isReadonly: true;
}

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type ReadonlyRecordStaticType<O extends RecordFields> = {
    readonly [K in keyof O]: Static<O[K]>;
};

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type ReadonlyStaticTuple<TElements extends readonly RuntypeBase<unknown>[]> = {
    readonly [key in keyof TElements]: TElements[key] extends RuntypeBase<infer E> ? E : unknown;
};

export declare function ReadonlyTuple<T extends readonly [RuntypeBase<unknown>, ...RuntypeBase<unknown>[]] | readonly []>(...components: T): ReadonlyTuple<T>;

export declare interface ReadonlyTuple<TElements extends readonly RuntypeBase<unknown>[] = readonly RuntypeBase<unknown>[]> extends Codec<ReadonlyStaticTuple<TElements>> {
    readonly tag: 'tuple';
    readonly components: TElements;
    readonly isReadonly: true;
}

/**
 * Construct a runtype for arbitrary dictionaries.
 */
declare function Record_2<K extends KeyRuntypeBase, V extends RuntypeBase<unknown>>(key: K, value: V): Record_2<K, V>;

declare interface Record_2<K extends KeyRuntypeBase, V extends RuntypeBase<unknown>> extends Codec<{
    [_ in Static<K>]?: Static<V>;
}> {
    readonly tag: 'record';
    readonly key: K;
    readonly value: V;
    readonly isReadonly: false;
}
export { Record_2 as MutableRecord }
export { Record_2 as Record }

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type RecordFields = {
    readonly [_: string]: RuntypeBase<unknown>;
};

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type RecordStaticType<O extends RecordFields, IsPartial extends boolean, IsReadonly extends boolean> = IsPartial extends false ? IsReadonly extends false ? MutableRecordStaticType<O> : ReadonlyRecordStaticType<O> : IsReadonly extends false ? PartialMutableRecordStaticType<O> : PartialReadonlyRecordStaticType<O>;

/**
 * The result of a type validation.
 */
export declare type Result<T> = Success<T> | Failure;

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type ResultWithCycle<T> = (Result<T> & {
    cycle?: false;
}) | Cycle<T>;

/**
 * A runtype determines at runtime whether a value conforms to a type specification.
 */
export declare interface Runtype<TParsed> extends RuntypeBase<TParsed> {
    /**
     * Union this Runtype with another.
     */
    Or<B extends RuntypeBase>(B: B): t.Union<[this, B]>;
    /**
     * Intersect this Runtype with another.
     */
    And<B extends RuntypeBase>(B: B): t.Intersect<[this, B]>;
    /**
     * Use an arbitrary constraint function to validate a runtype, and optionally
     * to change its name and/or its static type.
     *
     * @template T - Optionally override the static type of the resulting runtype
     * @param {(x: Static<this>) => boolean | string} constraint - Custom function
     * that returns `true` if the constraint is satisfied, `false` or a custom
     * error message if not.
     * @param [options]
     * @param {string} [options.name] - allows setting the name of this
     * constrained runtype, which is helpful in reflection or diagnostic
     * use-cases.
     */
    withConstraint<T extends Static<this>, K = unknown>(constraint: t.ConstraintCheck<this>, options?: {
        name?: string;
        args?: K;
    }): t.Constraint<this, T, K>;
    /**
     * Helper function to convert an underlying Runtype into another static type
     * via a type guard function.  The static type of the runtype is inferred from
     * the type of the test function.
     *
     * @template T - Typically inferred from the return type of the type guard
     * function, so usually not needed to specify manually.
     * @param {(x: Static<this>) => x is T} test - Type test function (see
     * https://www.typescriptlang.org/docs/handbook/advanced-types.html#user-defined-type-guards)
     *
     * @param [options]
     * @param {string} [options.name] - allows setting the name of this
     * constrained runtype, which is helpful in reflection or diagnostic
     * use-cases.
     */
    withGuard<T extends Static<this>, K = unknown>(test: (x: Static<this>) => x is T, options?: {
        name?: string;
        args?: K;
    }): t.Constraint<this, T, K>;
    /**
     * Adds a brand to the type.
     */
    withBrand<B extends string>(brand: B): t.Brand<B, this>;
    /**
     * Apply conversion functions when parsing/serializing this value
     */
    withParser<TParsed>(value: ParsedValueConfig<this, TParsed>): t.ParsedValue<this, TParsed>;
}

/**
 * A runtype determines at runtime whether a value conforms to a type specification.
 */
export declare interface RuntypeBase<TParsed = unknown> {
    readonly tag: string;
    /**
     * Verifies that a value conforms to this runtype. When given a value that does
     * not conform to the runtype, throws an exception.
     *
     * @throws ValidationError
     */
    assert(x: any): asserts x is TParsed;
    /**
     * A type guard for this runtype.
     */
    test(x: any): x is TParsed;
    /**
     * Validates the value conforms to this type, and performs
     * the `parse` action for any `ParsedValue` types.
     *
     * If the value is valid, it returns the parsed value,
     * otherwise it throws a ValidationError.
     *
     * @throws ValidationError
     */
    parse(x: any): TParsed;
    /**
     * Validates the value conforms to this type, and performs
     * the `parse` action for any `ParsedValue` types.
     *
     * Returns a `Result`, constaining the parsed value or
     * error message. Does not throw!
     */
    safeParse(x: any): Result<TParsed>;
    show?: (needsParens: boolean) => string;
    [internal]: InternalValidation<TParsed>;
}

declare const RuntypeName: unique symbol;

export declare function Sealed<TUnderlying extends RuntypeBase<unknown>>(underlying: TUnderlying, { deep }?: SealedConfig): Sealed<TUnderlying>;

export declare interface Sealed<TUnderlying extends RuntypeBase<unknown>> extends Codec<Static<TUnderlying>> {
    readonly tag: 'sealed';
    readonly underlying: TUnderlying;
    readonly deep: boolean;
}

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare interface SealedConfig {
    readonly deep?: boolean;
}

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type SealedState = {
    readonly keysFromIntersect?: ReadonlySet<string>;
    readonly deep: boolean;
} | false;

export declare function showError(failure: Omit<Failure, 'success'>): string;

export declare const showType: (runtype: RuntypeBase<unknown>, needsParens?: boolean) => string;

export declare function showValue(value: unknown, remainingDepth?: number, remainingLength?: number): string;

/**
 * Obtains the static type associated with a Runtype.
 */
export declare type Static<A extends RuntypeBase<any>> = A extends RuntypeBase<infer T> ? T : unknown;

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type StaticIntersect<TIntersectees extends readonly RuntypeBase<unknown>[]> = {
    [key in keyof TIntersectees]: TIntersectees[key] extends RuntypeBase ? (parameter: Static<TIntersectees[key]>) => any : unknown;
}[number] extends (k: infer I) => void ? I : never;

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type StaticTuple<TElements extends readonly RuntypeBase<unknown>[]> = {
    [key in keyof TElements]: TElements[key] extends RuntypeBase<infer E> ? E : unknown;
};

/**
 * This is an internal type that should not be referenced directly from outside of the funtypes package.
 * @internal
 */
export declare type StaticUnion<TAlternatives extends readonly RuntypeBase<unknown>[]> = {
    [key in keyof TAlternatives]: TAlternatives[key] extends RuntypeBase<unknown> ? Static<TAlternatives[key]> : unknown;
}[number];

declare interface String_2 extends Codec<string> {
    readonly tag: 'string';
}

/**
 * Validates that a value is a string.
 */
declare const String_2: String_2;
export { String_2 as String }

/**
 * A successful validation result.
 */
export declare type Success<T> = {
    /**
     * A tag indicating success.
     */
    success: true;
    /**
     * The original value, cast to its validated type.
     */
    value: T;
};

declare interface Symbol_2 extends Codec<symbol> {
    readonly tag: 'symbol';
}

/**
 * Validates that a value is a symbol.
 */
declare const Symbol_2: Symbol_2;
export { Symbol_2 as Symbol }

declare namespace t {
    export {
        AsyncContract,
        Contract,
        assertType,
        Runtype,
        RuntypeBase,
        Codec,
        Static,
        Success,
        Failure,
        Result,
        showError,
        ValidationError,
        showType,
        showValue,
        Readonly_2 as Readonly,
        Mutable,
        Arr as Array,
        Arr as MutableArray,
        ReadonlyArray_2 as ReadonlyArray,
        Obj as Object,
        Obj as MutableObject,
        ReadonlyObject,
        Partial_2 as Partial,
        Partial_2 as MutablePartial,
        ReadonlyPartial,
        Record_2 as Record,
        Record_2 as MutableRecord,
        ReadonlyRecord,
        Tuple,
        Tuple as MutableTuple,
        ReadonlyTuple,
        ConstraintCheck,
        Constraint,
        Guard,
        Enum,
        InstanceOf,
        Intersect,
        KeyOf,
        Lazy,
        LiteralValue,
        Literal,
        Null,
        Undefined,
        Named,
        Never,
        Boolean_2 as Boolean,
        Function_2 as Function,
        Number_2 as Number,
        String_2 as String,
        Symbol_2 as Symbol,
        BigInt_2 as BigInt,
        Sealed,
        Union,
        Unknown,
        Brand,
        ParsedValue
    }
}

/**
 * Construct a tuple runtype from runtypes for each of its elements.
 */
declare function Tuple<T extends readonly [RuntypeBase<unknown>, ...RuntypeBase<unknown>[]] | readonly []>(...components: T): Tuple<T>;

declare interface Tuple<TElements extends readonly RuntypeBase<unknown>[] = readonly RuntypeBase<unknown>[]> extends Codec<StaticTuple<TElements>> {
    readonly tag: 'tuple';
    readonly components: TElements;
    readonly isReadonly: false;
}
export { Tuple as MutableTuple }
export { Tuple }

/**
 * An alias for Literal(undefined).
 */
export declare const Undefined: Literal<undefined>;

/**
 * Construct a union runtype from runtypes for its alternatives.
 */
export declare function Union<TAlternatives extends readonly [RuntypeBase<unknown>, ...RuntypeBase<unknown>[]]>(...alternatives: TAlternatives): Union<TAlternatives>;

export declare interface Union<TAlternatives extends readonly RuntypeBase<unknown>[]> extends Codec<StaticUnion<TAlternatives>> {
    readonly tag: 'union';
    readonly alternatives: TAlternatives;
    match: Match<TAlternatives>;
}

export declare interface Unknown extends Codec<unknown> {
    readonly tag: 'unknown';
}

/**
 * Validates anything, but provides no new type information about it.
 */
export declare const Unknown: Unknown;

export declare class ValidationError extends Error {
    name: string;
    readonly shortMessage: string;
    readonly key: string | undefined;
    readonly fullError: FullError | undefined;
    constructor(failure: Omit<Failure, 'success'>);
}

export { }
